/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: f_gplot.cpp,v 1.7 1997/06/24 17:43:24 reichel Exp $ *
 ***********************************************************************/

/* ANSI and traditional C compatability macros
   Copyright 1991, 1992 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */


/*
 * File:    f_gplot.cpp
 * Purpose: module to generate evaluation file for gnuplot.
 * Author:  Stefan Mayer
 */
#include <string.h>
#include <strings.h>
#include "flip.h"
#include <stream.h>
#include <errno.h>

#include "wx.h"
#include "wx_help.h"
#include "f_gplot.h"


FrameFlipGplot::FrameFlipGplot(wxFrame *parent, char *title, int x, int y, int width, int height, long style, char *name):
  wxFrame(parent, title, x, y, width, height, style, name)
{
  // Initialize child subwindow members.
  panel8 = NULL;

  // Create child subwindows.
  panel8 = new wxPanel(this, 0, 0, 564, 563, wxBORDER);
  

  mess_top = new wxMessage(panel8,"select a output variable and one or two input variables:",38,13);
  input1 = new wxChoice(panel8,(wxFunction)&fgp_v1_call,"input1:",13,70,-1,-1);
  input2 = new wxChoice(panel8,(wxFunction)&fgp_v2_call,"input2:",316,70,-1,-1);
  output_variable = new wxChoice(panel8,(wxFunction)&fgp_outp_call,"output:",160,33,-1,-1);
  listbox_input = new wxListBox(panel8,(wxFunction)&fgp_const_list,"constant input variables",wxSINGLE,160,220,-1,60);
  const_value = new wxText(panel8,(wxFunction)&fgp_const_value,"constant value =","",160,330);

  cb_gen_dat= new wxCheckBox(panel8,(wxFunction)&cb_gen_dat_call,"generate data file --> *.dat",20,360);
  cb_gen_dat->SetValue(TRUE);
 cb_gen_eps= new wxCheckBox(panel8,(wxFunction)&cb_gen_gpp_call,"generate gnuplot postscript program and a postscript file --> *.gpp --> *.eps ",20,390);
 // ggp .. gnuplot graphic program
 cb_gen_ggp= new wxCheckBox(panel8,NULL,"generate gnuplot graphic program --> *.ggp",20,420);
 cb_gen_ghs= new wxCheckBox(panel8,(wxFunction)&cb_gen_ghs_call,"execute ghostscript",20,450);

  ok_button = new wxButton(panel8,(wxFunction)&fgp_ok_button,"OK",406,533);
  cancel_button = new wxButton(panel8,(wxFunction)&fgp_cancel_button,"CANCEL",466,533);
  text_v1_from = new wxText(panel8,NULL,"from =","",15,102);
  text_v1_to = new wxText(panel8,NULL,"to =","",34,136);
  text_v1_step = new wxText(panel8,NULL,"step =","",18,170);
  text_v2_from = new wxText(panel8,NULL,"from =","",315,113);
  text_v2_to = new wxText(panel8,NULL,"to =","",334,149);
  text_v2_step = new wxText(panel8,NULL,"step =","",319,182);

  file_button = new wxButton(panel8,(wxFunction)&call_file_button,"file",16,510);
  text_file = new wxText(panel8,NULL,"file =","",16,541,190,-1,wxREADONLY);

  // Ensure the subwindows get resized o.k.
  OnSize(width, height);

  // Centre frame on the screen.
  Centre(wxBOTH);


  // Show the frame.
  // Show(TRUE);

  last_const_cv_ = NULL;
}

FrameFlipGplot::~FrameFlipGplot(void)
{     
  signal(SIGTERM,SIG_IGN);// ignore SIGTERM
  if(kill(0,SIGTERM)==-1)  // kill process group
    cout << "error: can't kill process-group(pgid = "<<getpgrp()<<")\n";
  else cout << " process-group killed (pgid = " << getpgrp() << ")\n";

  //gnu_pid_list.Kill_all();
}
void FrameFlipGplot::Show(RuleSet *r)
{ 
  rule_set_ = r;
  if(r) {
    cv_list_ = r->get_cv_list();
    set_output_list();
    set_v1_list();
    set_v2_list();
    set_const_v_list();
    wxFrame::Show(TRUE);
  }
  else wxMessageBox("Sorry, but there are no rules ..","Nothing to evaluate",wxOK);
}

Bool FrameFlipGplot::OnClose(void)
{
  // Clean up any associated data here.
  //theApp.frame5 = NULL;
  return TRUE;
}

void FrameFlipGplot::OnSize(int w, int h)
{
  // Let default member handle sizing for only one subwindow.
  wxFrame::OnSize(w, h);
}

void FrameFlipGplot::OnMenuCommand(int commandId)
{
  switch (commandId)
  {
  default:
    break;
  }
}

void FrameFlipGplot::set_output_list(void)
{
  output_variable->Clear(); // xview needs a refreshing of the window
  // old version:
  //  if (cv_list)
  //  {
  //    for(int nr=0; nr<cv_list->get_number(); nr++)
  //      output_variable->Append(cv_list->find(nr)->get_cv_name());
  //  }

  CombinedVariable *cv_x;
  for(int no_rb=0;no_rb<rule_set_->get_number();no_rb++){
    cv_x = rule_set_->get_right_side_cv(no_rb,0);
    if(cv_x)
      output_variable->Append(cv_x->get_cv_name());
  }
}
void FrameFlipGplot::set_v1_list(name cv_out)
  // show all 'basic' input variables of 
  //  -1- cv_out or
  //  -2- the selected output variable in the wxChoiceBox 'output_variable'
  //
  // i.e. all left side variables of rule with cv_out at the right side, 
  //      that do not exist at the right side of a rule
{  
  /*
  if (cv_list==NULL) cv_list = cv_list_;
  input1->Clear(); // xview needs a refreshing of the window
  name var_name;
  char *v_name; 
  strcpy(var_name, output_variable->GetStringSelection());
  if (cv_list)
  {
    for(int nr=0; nr<cv_list->get_number(); nr++){
      v_name = cv_list->find(nr)->get_cv_name();
      if(strcmp(v_name,var_name))
	input1->Append(v_name);
    }
    update_v1_fts();
  }
  */
  char *var_ptr;
  name var_name,out_name;
  if (strlen(cv_out)==0){
    input1->Clear();
    strcpy(out_name, output_variable->GetStringSelection());
  }
  else strcpy(out_name,cv_out);
  int no_rb=get_rule_block_no(out_name);
  if (no_rb != -1)
    for(int no_rule=0;no_rule<rule_set_->get_rule_number(no_rb);no_rule++){
      for(int no_op=0;no_op<rule_set_->get_operand_number(no_rb,no_rule);no_op++){
	var_ptr = rule_set_->get_left_side_cv(no_rb,no_rule,no_op)->get_cv_name(); // input variable
	strcpy(var_name,var_ptr);
	if (get_rule_block_no(var_name)== -1)
	  if(input1->FindString(var_name)==-1)
	    input1->Append(var_ptr);
	  else set_v1_list(var_name);
      }
    }
  update_v1_fts();
}

int FrameFlipGplot::get_rule_block_no(name cv_name)
{
  CombinedVariable *cv_x;
  if(strlen(cv_name)){ // find ruleblock for cv_name:
    for(int no_rb=0;no_rb<rule_set_->get_number();no_rb++){
      cv_x = rule_set_->get_right_side_cv(no_rb,0);
      if(cv_x)
	{
	if(strcmp(cv_name,cv_x->get_cv_name())==0)  // right ruleblock!
	  return no_rb;
	}
    }}
  return -1;
}


void FrameFlipGplot::set_v2_list(void)
{
  //take all variables shown in input1 listbox
  // exept the selected one
  name old_name_selected;
  int old_name_selected_no = -1;
  int i=0;
  strcpy(old_name_selected,input2->GetStringSelection());
  input2->Clear();
  name input1_name;
  char *input1_ptr = input1->GetStringSelection();
  if (input1_ptr){
    strcpy(input1_name,input1_ptr);
    name var_name;
    for (int nr=0;nr<input1->Number();nr++)
      {
	strcpy(var_name,input1->GetString(nr));
	if(strcmp(var_name,input1_name)){
	  input2->Append(var_name);
	  i++;
	  if(strcmp(var_name,old_name_selected)==0)
	    old_name_selected_no = i-1;
	}
      }
  }
  input2->Append("NONE");
  // select old name if possible:
  if (old_name_selected_no > -1)
    input2->SetSelection(old_name_selected_no);
  else
    update_v2_fts();
}

void FrameFlipGplot::set_const_v_list(void)
{ 
  // set all variables shown in input1 list 
  // exept the selected input1 and input2 variables 
  listbox_input->Clear();
  name input1_name,input2_name;
  strcpy(input1_name,input1->GetStringSelection());
  strcpy(input2_name,input2->GetStringSelection());
  name var_name;
  for (int nr=0;nr<input1->Number();nr++)
  {
    strcpy(var_name,input1->GetString(nr));
    if(strcmp(var_name,input1_name)&&strcmp(var_name,input2_name))
      listbox_input->Append(var_name);
  }
  show_const_value();
}

// read data in window and call evaluate_list(char* ..)
Bool FrameFlipGplot::evaluate_list(void)
{
  if(!cv_list_) return FALSE;
  name var_name;
  char textvalue[50];

  // output CombinedVariable
  CombinedVariable *out_cv;
  strcpy(var_name, output_variable->GetStringSelection());
  out_cv = cv_list_->find_cv(var_name);

  if(output_variable->Number() == 0) {// no output variable exists
    wxMessageBox("There are no output variables.\n","Sorry, evaluation failed",wxOK);
    return FALSE;
  }

  if(input1->Number() == 0){ // no input variable e.g. no rule for output variable
    wxMessageBox("There are no rules for the chosen output variable.\n","Sorry, evaluation failed",wxOK);
    return FALSE;
  }
  //FILE NAME:
  char filename[200];
  strncpy(filename,text_file->GetValue(),199);
  if (!filename || (strcmp(filename,"")==0)) {
    wxMessageBox("Input a filename, please.","Sorry, evaluation failed",wxOK);
    return FALSE;
  }
  //cout << "file = "<<filename<<" ("<<strlen(text_file->GetValue())<<")\n";
  filename[strlen(text_file->GetValue())] = '\0'; // strncpy ..

  // eliminate starting '/tmp_mnt/..'
  const char start[10]="/tmp_mnt";
  if(strncmp(filename,start,8)==0)
    strcpy(filename,filename+8);

  // input variable v1:
  CombinedVariable *v1_cv;
  float v1_from,v1_to,v1_step;
  strcpy(var_name, input1->GetStringSelection());
  v1_cv = cv_list_->find_cv(var_name);
  //       from:
  strncpy(textvalue,text_v1_from->GetValue(),49);
  StringToFloat(textvalue,&v1_from);
  if (!check_from(v1_from,v1_cv)) return FALSE; 
  //       to:
  strncpy(textvalue,text_v1_to->GetValue(),49);
  StringToFloat(textvalue,&v1_to);
  if (!check_to(v1_to,v1_cv)) return FALSE;
  //       step:
  strncpy(textvalue,text_v1_step->GetValue(),49);
  StringToFloat(textvalue,&v1_step);
  if (!check_step(v1_step,v1_cv,v1_from,v1_to)) return FALSE;

  // input variable v2 ?:
  CombinedVariable *v2_cv;
  float v2_from,v2_to,v2_step;
  strcpy(var_name, input2->GetStringSelection());
  v2_cv = cv_list_->find_cv(var_name);
  if(v2_cv) {
    //       from:
    strncpy(textvalue,text_v2_from->GetValue(),49);
    StringToFloat(textvalue,&v2_from);
    if (!check_from(v2_from,v2_cv)) return FALSE;
    //       to:
    strncpy(textvalue,text_v2_to->GetValue(),49);
    StringToFloat(textvalue,&v2_to);
    if (!check_to(v2_to,v2_cv)) return FALSE;
    //       step:
    strncpy(textvalue,text_v2_step->GetValue(),49);
    StringToFloat(textvalue,&v2_step);
    if (!check_step(v2_step,v2_cv,v2_from,v2_to)) return FALSE;

    return evaluate_list(out_cv,v1_cv,v1_from,v1_to,v1_step,v2_cv,v2_from,v2_to,v2_step,filename);
  }
  else return evaluate_list(out_cv,v1_cv,v1_from,v1_to,v1_step,filename);
}


Bool FrameFlipGplot::evaluate_list(CombinedVariable* out_cv,CombinedVariable *v1_cv, float v1_from,float v1_to,float v1_step, CombinedVariable* v2_cv, float v2_from, float v2_to, float v2_step, char* file)
{
  if(!out_cv || !v1_cv || !v2_cv) return FALSE;
  ofstream output_file(file,ios::out);
  if(!output_file)
    {
      wxMessageBox("cannot open output file","evaluation failed",wxOK);
      return FALSE;
    }

  // header for output_file:
  name tmp_name;
  char str_val[40];
  CombinedVariable *cv;
  output_file<< "# constant values:\n";
  for(int nr=0;nr<listbox_input->Number();nr++){
    strcpy(tmp_name,listbox_input->GetString(nr));
    if(strcmp(tmp_name,v1_cv->get_cv_name())&&
	      strcmp(tmp_name,v2_cv->get_cv_name())){
      cv = cv_list_->find_cv(tmp_name);
      strncpy(str_val,FloatToString(cv->get_crisp_value()),39);
      output_file<<"#     "<<tmp_name<<" = "<<str_val<<"\n";
    }
  }
  output_file <<"#input 1"<<"   "<<
    "input2"<<"   "<<
    "output"<<"\n";

  output_file <<"#"<<v1_cv->get_cv_name()<<"   "<<
    v2_cv->get_cv_name()<<"   "<<
    out_cv->get_cv_name()<<"\n#------------------------------------\n";

  // generate evaluation-list:
  char v1_value_str[30],v2_value_str[30],out_value_str[30];
  float v1_value, v2_value, out_evaluation_value;
  for(v1_value=v1_from;v1_value<=v1_to;v1_value+=v1_step)
    {
      v1_cv->set_value(v1_value);
      strcpy(v1_value_str,FloatToString(v1_value));
      for(v2_value=v2_from;v2_value<=v2_to;v2_value+=v2_step)
	{
	v2_cv->set_value(v2_value);
	strcpy(v2_value_str,FloatToString(v2_value));
	out_evaluation_value = rule_set_->evaluate(out_cv);
	strcpy(out_value_str,FloatToString(out_evaluation_value));
	output_file << v1_value_str<< "  "<< v2_value_str << "  " <<
	  out_value_str<<"\n";
	}
      output_file <<"\n";
    }
  wxMessageBox("OK - evaluation completed","OK",wxOK);
  if (cb_gen_eps->GetValue())
    execute_eps(file,v1_cv->get_cv_name(),v2_cv->get_cv_name(),out_cv->get_cv_name());
  if (cb_gen_ggp->GetValue())
    execute_ggp(file,v1_cv->get_cv_name(),v2_cv->get_cv_name(),out_cv->get_cv_name());
  if (cb_gen_ghs->GetValue())  // ghostscript
    execute_ghs(file);
  return TRUE;
}

Bool FrameFlipGplot::evaluate_list(CombinedVariable* out_cv,CombinedVariable *v1_cv, float v1_from,float v1_to,float v1_step,char* file)
{
  if(!out_cv || !v1_cv) return FALSE;
  ofstream output_file(file,ios::out);
  if(!output_file)
    {
      wxMessageBox("cannot open output file","evaluation failed",wxOK);
      return FALSE;
    }
  // header for output_file:
  name tmp_name;
  CombinedVariable *cv;
  char str_val[40];
  output_file<< "# constant values:\n";
  for(int nr=0;nr<listbox_input->Number();nr++){
    strcpy(tmp_name,listbox_input->GetString(nr));
    if(strcmp(tmp_name,v1_cv->get_cv_name())){
      cv = cv_list_->find_cv(tmp_name);
      strncpy(str_val,FloatToString(cv->get_crisp_value()),39);
      output_file<<"#     "<<tmp_name<<" = "<<str_val<<"\n";
    }
  }
  output_file <<"#input 1"<<"   "<<
    "output"<<"\n";

  output_file <<"#"<<v1_cv->get_cv_name()<<"   "<<
    out_cv->get_cv_name()<<"\n#------------------------------------\n";


  char v1_value_str[30],out_value_str[30];
  float v1_value, out_evaluation_value;
  for(v1_value=v1_from;v1_value<=v1_to;v1_value+=v1_step)
    {
      v1_cv->set_value(v1_value);
      strcpy(v1_value_str,FloatToString(v1_value));
      out_evaluation_value = rule_set_->evaluate(out_cv);
      strcpy(out_value_str,FloatToString(out_evaluation_value));
      output_file << v1_value_str<< "  " << out_value_str<<"\n";
    }
  wxMessageBox("OK - evaluation completed","OK",wxOK);

  if (cb_gen_eps->GetValue())
    execute_eps(file,v1_cv->get_cv_name(),out_cv->get_cv_name());
  if (cb_gen_ggp->GetValue())
    execute_ggp(file,v1_cv->get_cv_name(),out_cv->get_cv_name());
  if (cb_gen_ghs->GetValue())  // ghostscript
    execute_ghs(file);
  return TRUE;
}


Bool FrameFlipGplot::execute_eps(char *file,char *x_var,char *y_var,char *z_var)
  // z_var .. NULL --> two dimensional graph
  // file .. path and name of *.dat file
  // create a postscriptfile executing gnuplot
{
  long pid;
  char filename[130];
  char file_out[130];

  char *str_ptr = rindex(file,'.');
  int no2 = strlen(str_ptr);
  int no1 = strlen(file);
  strncpy(filename,file,no1-no2);
  filename[no1-no2] = '\0';
  //cout << "eps-file = " << filename <<" -- "<< no2 <<","<<no1<<"\n";

  strcpy(file_out,filename);
  strcat(file_out,".gpp"); // GNUPLOT POSTSCRIPT PROGRAM
  ofstream gpp_file(file_out,ios::out);
  if(!gpp_file)
  {
    wxMessageBox("cannot open *.gpp file","evaluation failed",wxOK);
    return FALSE;
  };
  if(z_var){
// three dimensional graph
    gpp_file << "set parametric\nset contour\nset view 60,70\n";
    gpp_file << "set xlabel \"" << x_var <<"\"\n"; 
    gpp_file << "set ylabel \"" << y_var <<"\"\n"; 
    gpp_file << "set zlabel \"" << z_var <<"\"\n"; 
    gpp_file << "set term postscript eps color 14\n";
    gpp_file << "set output \"" << filename << ".eps\"\n";
    gpp_file << "splot \"" << file << "\" using 1:2:3 with lines\n";
  }
  else // 2 dim
    {
      gpp_file << "set term postscript eps color 14\n";
      gpp_file << "set output \"" << filename << ".eps\"\n";
      gpp_file << "set xlabel \"" << x_var <<"\"\n"; 
      gpp_file << "set ylabel \"" << y_var <<"\"\n"; 
      gpp_file << "plot \"" << file << "\" using 1:2:3 with lines\n";      
    }
  
  // run gnuplot 
  char str_x[120];
  strcpy(str_x,"gnuplot ");
  strcat(str_x,filename);
  strcat(str_x,".gpp");
  pid = wxExecute(str_x,FALSE);
  cout << "gnuplot -  pid = "<< pid <<" !\n";

  return TRUE;
}
Bool FrameFlipGplot::execute_ggp(char *file,char *x_var,char *y_var,char *z_var)
  // z_var .. NULL --> two dimensional graph
  // file .. path and name of *.dat file
  // execute gnuplot to show graph
{
  long pid;
  char filename[130];
  char file_out[130];

  char *str_ptr = rindex(file,'.');
  int no2 = strlen(str_ptr);
  int no1 = strlen(file);
  strncpy(filename,file,no1-no2);
  filename[no1-no2] = '\0';

  strcpy(file_out,filename);
  strcat(file_out,".ggp");   // GNUPLOT GRAPHIC PROGRAM
  ofstream ggp_file(file_out,ios::out);
  if(!ggp_file)
  {
    wxMessageBox("cannot open *.ggp file","evaluation failed",wxOK);
    return FALSE;
  };
  if(z_var){
// three dimensional graph
    ggp_file << "set parametric\nset contour\nset view 60,70\n";
    ggp_file << "set xlabel \"" << x_var <<"\"\n"; 
    ggp_file << "set ylabel \"" << y_var <<"\"\n"; 
    ggp_file << "set zlabel \"" << z_var <<"\"\n"; 
    ggp_file << "splot \"" << file << "\" using 1:2:3 with lines\n";
    ggp_file << "pause 60\n";
  }
  else // 2 dim
    {
      ggp_file << "set xlabel \"" << x_var <<"\"\n"; 
      ggp_file << "set ylabel \"" << y_var <<"\"\n"; 
      ggp_file << "plot \"" << file << "\" using 1:2:3 with lines\n";      
      ggp_file << "pause 35000\n";// terminates after 35000 sec.
    }
  
  // run gnuplot 
  char str_x[120];

//  strcpy(str_x,"test");// TEST: sonst strcpy(str_x,filename);
//  strcat(str_x,".ggp");
//  char *argv[3];
//  argv[0] = "gnuplot";
//  argv[1] = str_x;
//  argv[2] = NULL;
//  pid = Execute_test(argv);

  // suspended (tty input) for flip !!  if gnuplot uses pause -1 !!!!
  strcpy(str_x,"gnuplot ");
  strcat(str_x,filename);
  strcat(str_x,".ggp");
  pid = wxExecute(str_x,FALSE);
  //if (pid) gnu_pid_list.Append(pid);
  cout << "show gnuplot -  pid = " << pid << " !\n";
  return TRUE;
}

long FrameFlipGplot::Execute_test (char **argv)
{
  if (*argv == NULL)
    return 0;	// Nothing???
  // Run a program the recomended way under X (XView) 

  /* fork the process */
#if defined(sun) || defined(__ultrix) || defined(__bsdi__)
//  pid_t pid = vfork ();
  pid_t pid = fork ();
#else
  pid_t pid = fork ();
#endif
  if (pid == -1)
    {
      return 0;
    }
  else if (pid == 0)
    {
      /* child */
#ifdef _AIX
      execvp ((const char *)*argv, (const char **)argv);
#else
      execvp (*argv, argv);
#endif
      if (errno == ENOENT)
	printf ("%s: command not found\n", *argv);
      else
	perror (*argv);
      printf ("wxWindows: could not execute '%s'\n", *argv);
      _exit (-1);
    }

//#ifdef wx_xview
//  static Notify_client sys_client = 42;
//  notify_set_wait3_func (sys_client, (Notify_func) notify_default_wait3, pid);
//#endif


  return pid;
}

Bool FrameFlipGplot::execute_ghs(char *file)
  // run ghostview
{
  char filename[130];

  char *str_ptr = rindex(file,'.');
  int no2 = strlen(str_ptr);
  int no1 = strlen(file);
  strncpy(filename,file,no1-no2);
  filename[no1-no2] = '\0';

  char str_x[120];
  strcpy(str_x,"ghostview ");
  strcat(str_x,filename);
  strcat(str_x,".eps");
  long pid = wxExecute(str_x,FALSE);
  //if(pid) gnu_pid_list.Append(pid);
  cout << "ghostview - pid = "<< pid <<" !\n";
  return TRUE;
}

void FrameFlipGplot::set_default_v1_fts(CombinedVariable* var)
{
  if(var){
    float from = var->get_fs_lower_support_end(0);
    float to = var->get_fs_upper_support_end(var->get_number()-1);
    float step = (to - from)/ 10;
    char str_from[40],str_to[40],str_step[40];
    strcpy(str_from,FloatToString(from));
    strcpy(str_to,FloatToString(to));
    strcpy(str_step,FloatToString(step));
    text_v1_from->SetValue(str_from);
    text_v1_to->SetValue(str_to);
    text_v1_step->SetValue(str_step);
  }
  else {
    text_v1_from->SetValue("");
    text_v1_to->SetValue("");
    text_v1_step->SetValue("");
  }
}
void FrameFlipGplot::set_default_v2_fts(CombinedVariable* var)
{
  if(var){
    float from = var->get_fs_lower_support_end(0);
    float to = var->get_fs_upper_support_end(var->get_number()-1);
    float step = (to - from)/ 10;
    char str_from[40],str_to[40],str_step[40];
    strcpy(str_from,FloatToString(from));
    strcpy(str_to,FloatToString(to));
    strcpy(str_step,FloatToString(step));
    text_v2_from->SetValue(str_from);
    text_v2_to->SetValue(str_to);
    text_v2_step->SetValue(str_step);
  }
  else {
    text_v2_from->SetValue("");
    text_v2_to->SetValue("");
    text_v2_step->SetValue("");
  }
}

void FrameFlipGplot::update_v1_fts(void)
{
  name str_val;
  strcpy(str_val,"");
  if(cv_list_){
    name var_name;
    strcpy(var_name, input1->GetStringSelection());
    CombinedVariable *cv = cv_list_->find_cv(var_name);
    if (cv)
      set_default_v1_fts(cv);
  }
}
void FrameFlipGplot::update_v2_fts(void)
{
  name str_val;
  strcpy(str_val,"");
  if(cv_list_){
    name var_name;
    strcpy(var_name, input2->GetStringSelection());
    CombinedVariable *cv = cv_list_->find_cv(var_name);
    set_default_v2_fts(cv);
  }
}

CombinedVariable *FrameFlipGplot::get_selected_const_cv(void)
{
  if(cv_list_){
    name var_name;
    char *sel_cv_ptr = listbox_input->GetStringSelection();
    if (sel_cv_ptr)
    strcpy(var_name, sel_cv_ptr);
    else return NULL;
    return cv_list_->find_cv(var_name);
  }
  return NULL;
}

void FrameFlipGplot::show_const_value(void)
{
  char str_val[50];
  strcpy(str_val,"");
  if(cv_list_){
    float val = 0;
    CombinedVariable *cv = get_selected_const_cv();
    if(cv) {
      val = cv->get_crisp_value();
      strcpy(str_val,FloatToString(val));
    }
  }
  const_value->SetValue(str_val);
}

void FrameFlipGplot::set_cv_value(Bool take_selected_cv)
{
  char str_value[50];
  CombinedVariable *cv;
  float crisp_value;

  // get value in input_text_area(wxText):
  strcpy(str_value,const_value->GetValue());
  StringToFloat(str_value,&crisp_value);
  
  if(take_selected_cv) {
    // get selected CombinedVariable in ListBox
    cv = get_selected_const_cv();
  }
  else {
     cv = last_const_cv_;
  }
  // set value

  if(cv && check_crisp_value(crisp_value,cv)) {
    cv->set_value(crisp_value);
    last_const_cv_ = get_selected_const_cv();
  }
}

Bool FrameFlipGplot::check_crisp_value(float value, CombinedVariable* cv)
{
  char mess_str[100],range_str[20];
  strcpy(mess_str,"The crisp value of ");
  strcat(mess_str,cv->get_cv_name());
  float from = cv->get_fs_lower_support_end(0);
  float to = cv->get_fs_upper_support_end(cv->get_number()-1);
  if (value<from){
    strcat(mess_str," is smaller than the lower support range (");
    strncpy(range_str,FloatToString(from),19);
    strcat(mess_str,range_str);
    strcat(mess_str,").");
    wxMessageBox(mess_str);
    return FALSE;
  }
  if (value>to){
    strcat(mess_str," is greater than the upper support range (");
    strncpy(range_str,FloatToString(to),19);
    strcat(mess_str,range_str);
    strcat(mess_str,").");
    wxMessageBox(mess_str);
    return FALSE;
  }
  return TRUE;
}
 Bool FrameFlipGplot::check_from(float num,CombinedVariable *cv)
{
  float from = cv->get_fs_lower_support_end(0);
  if(num<from) {
    char mess_str[100],range_str[20];
    strcpy(mess_str,"The from-value of ");
    strcat(mess_str,cv->get_cv_name());
    strcat(mess_str," is smaller than the lower support range(");
    strncpy(range_str,FloatToString(from),19);
    strcat(mess_str,range_str);
    strcat(mess_str,").");
    wxMessageBox(mess_str);
    return FALSE;
  }
  return TRUE;
}

 Bool FrameFlipGplot::check_to(float num,CombinedVariable *cv)
{
  float to = cv->get_fs_upper_support_end(cv->get_number()-1);
  if(num>to) {
    char mess_str[100],range_str[20];
    strcpy(mess_str,"The to-value of ");
    strcat(mess_str,cv->get_cv_name());
    strcat(mess_str," is greater than the upper support range (");
    strncpy(range_str,FloatToString(to),19);
    strcat(mess_str,range_str);
    strcat(mess_str,").");
    wxMessageBox(mess_str);
    return FALSE;
  }
  return TRUE;
}

 Bool FrameFlipGplot::check_step(float step,CombinedVariable *cv,float from ,float to)
{
  if (step > (to-from)){
    char mess_str[100];
    strcpy(mess_str,"The step-value of ");
    strcat(mess_str,cv->get_cv_name());
    strcat(mess_str," is too big.");
    wxMessageBox(mess_str);
    return FALSE;
  }
  if(from>to){
    char mess_str[100];
    strcpy(mess_str,"The from-value of ");
    strcat(mess_str,cv->get_cv_name());
    strcat(mess_str," is greater than the to-value.");
    wxMessageBox(mess_str);
    return FALSE;
  }
  return TRUE;
}

void FrameFlipGplot::get_filename(void)
{
  char *filename = wxFileSelector("choose a  data file",
				  NULL,NULL,"*.dat","*.dat");
  text_file->SetValue(filename);
}

 void fgp_const_list (wxListBox& o, wxCommandEvent&)
{
  FrameFlipGplot *win = (FrameFlipGplot *)o.GetGrandParent();
  win->set_cv_value(FALSE);
  win->show_const_value();
};

 void fgp_v1_call (wxChoice& o, wxCommandEvent& )
{ 
  FrameFlipGplot *win = (FrameFlipGplot *)o.GetGrandParent();
  win->set_cv_value();
  win->update_v1_fts();
  win->set_v2_list();
  win->update_v2_fts();
  win->set_const_v_list();
};

 void fgp_v2_call (wxChoice& o, wxCommandEvent& )
{ 
  FrameFlipGplot *win = (FrameFlipGplot *)o.GetGrandParent();
  win->set_cv_value();
  win->update_v2_fts();
  win->set_const_v_list();
};

 void fgp_outp_call (wxChoice& o, wxCommandEvent&)
{
  FrameFlipGplot *win = (FrameFlipGplot *)o.GetGrandParent();
  win->set_cv_value();
  win->set_v1_list();
  win->set_v2_list();
  win->set_const_v_list();
};

 void fgp_const_value (wxText& o, wxCommandEvent &event)
{
  if(event.eventType==wxEVENT_TYPE_TEXT_ENTER_COMMAND){
    FrameFlipGplot *win = (FrameFlipGplot *)o.GetGrandParent();
    win->set_cv_value();
  }
};
 void call_file_button (wxButton& o, wxCommandEvent&)
{
  FrameFlipGplot *win = (FrameFlipGplot *)o.GetGrandParent();
  win->get_filename();
};
 void fgp_ok_button (wxButton& o, wxCommandEvent&)
{
  FrameFlipGplot *win = (FrameFlipGplot *)o.GetGrandParent();
  win->set_cv_value();
  win->evaluate_list();
};
 void fgp_cancel_button (wxButton& o, wxCommandEvent&)
{
  FrameFlipGplot *win = (FrameFlipGplot *)o.GetGrandParent();
  win->wxFrame::Show(FALSE);
};
void cb_gen_dat_call (wxCheckBox& o,wxCommandEvent&)
{
    o.SetValue(TRUE);
}

// callback of ghostscript checkbox:
void cb_gen_ghs_call (wxCheckBox& o,wxCommandEvent&)
{
    FrameFlipGplot *win = (FrameFlipGplot *)o.GetGrandParent();
    win->cb_gen_eps->SetValue(TRUE);
}
void cb_gen_gpp_call (wxCheckBox& o,wxCommandEvent&)
{
    FrameFlipGplot *win = (FrameFlipGplot *)o.GetGrandParent();
    if(win->cb_gen_ghs->GetValue()) o.SetValue(TRUE);
}






listOfLong::~listOfLong(void)
{
  Kill_all();
}

void listOfLong::Append(long nr)
{
  if (next == NULL)
    {
      listOfLong *new_member = new listOfLong();
      new_member->no_ = nr;
      next = new_member;
    }
  else next->Append(nr);
}
void listOfLong::Kill_all(void)
{
  if(next!=NULL) {
    next->Kill_all();
    if(kill(next->no_,SIGKILL)==-1)  // kill process
      cout << "error: can't kill process (pid = "<<next->no_<<")\n";
    else cout << " process killed (pid = " << next->no_ << ")\n";
    delete next;
    next = NULL;
  }
}
