/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: f_gplot.h,v 1.6 1997/06/24 17:43:25 reichel Exp $ *
 ***********************************************************************/

/* ANSI and traditional C compatability macros
   Copyright 1991, 1992 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*
 * File:    f_gplot.h
 * Purpose: generate file as input for gnuplot
 * Author:  Stefan Mayer 
 */

#ifndef _INC_NONAME_H
#define _INC_NONAME_H


class listOfLong
{
 private:
  long no_;
  listOfLong *next;
 public:
  listOfLong(void){no_ = 0;next = NULL;};
  ~listOfLong(void);
  void Append(long);
  void Kill_all(void);
};

class PanelFlipGplot;

class FrameFlipGplot: public wxFrame
{
  friend void cb_gen_ghs_call (wxCheckBox& o,wxCommandEvent&);
  friend void cb_gen_gpp_call (wxCheckBox& o,wxCommandEvent&);
 private:
  //listOfLong gnu_pid_list;
  wxListBox *listbox_input; // listbox contains constant cv
  wxMessage *mess_top;
  wxChoice  *input1, *input2, *output_variable;
  wxText *const_value,*text_v1_from,*text_v1_to,*text_v1_step,*text_v2_from,*text_v2_to,*text_v2_step,*text_file;
  wxCheckBox *cb_gen_dat, *cb_gen_eps, *cb_gen_ggp, *cb_gen_ghs;
  wxButton *ok_button, *cancel_button, *file_button;
  
 protected:
  CVList *cv_list_;
  RuleSet *rule_set_;
  CombinedVariable *last_const_cv_; // last constant variable in constant cv listbox

 public:
  // Subwindows for reference within the program.
  wxPanel *panel8;

  // Constructor and destructor
  FrameFlipGplot(wxFrame *parent, char *title, int x, int y, int width, int height, long style, char *name);
  ~FrameFlipGplot(void);

 Bool OnClose(void);
 void OnSize(int w, int h);
 void OnMenuCommand(int commandId);

 void Show(RuleSet *l);


//set list and choice boxes:
 void set_output_list(void);
 void set_v1_list(name = ""); // set first input  variable choice box
 void set_v2_list(void); // set 2nd input variable choice box
 void set_const_v_list(void);

// evaluate list:
 Bool evaluate_list(CombinedVariable* out_cv, CombinedVariable* v1_cv, float v1_from, float v1_to, float v1_step, CombinedVariable* v2_cv, float v2_from, float v2_to, float v2_step, char* file);
 Bool evaluate_list(CombinedVariable* out_cv, CombinedVariable* v1_cv, float v1_from, float v1_to, float v1_step, char* file);
 Bool evaluate_list(void);
 Bool execute_eps(char* file,char *,char*,char* z_var=NULL);
 Bool execute_ggp(char* file,char *,char*,char* z_var=NULL);
 Bool execute_ghs(char* file);
 long Execute_test (char **argv);

 // set from-to-step (fts):
 void set_default_v1_fts(CombinedVariable*);
 void set_default_v2_fts(CombinedVariable*);
 void update_v1_fts(void); // like set_default_v1_fts, but for selected cv
 void update_v2_fts(void); // like set_default_v2_fts, but for selected cv
 
// miscellaneous:
 int get_rule_block_no(name cv_name);
 void get_filename(void); // input a filename an write it to wxText
 void set_cv_value(Bool take_selected_cv = TRUE); // set a constant input value for the selected variable in constant_variable_list	
 void show_const_value(void);
 CombinedVariable *get_selected_const_cv(void);
 Bool check_crisp_value(float,CombinedVariable *);
 Bool check_from(float,CombinedVariable *);
 Bool check_to(float,CombinedVariable *); 
 Bool check_step(float,CombinedVariable *,float,float);
};


/*
set (1) output variable
    (2) input variables
    (3) evaluate 
*/


// callback functions:
 void call_file_button (wxButton& , wxCommandEvent&);
 void fgp_const_list (wxListBox& , wxCommandEvent&);
 void fgp_v1_call (wxChoice& , wxCommandEvent&);
 void fgp_v2_call (wxChoice& , wxCommandEvent&);
 void fgp_outp_call (wxChoice& , wxCommandEvent&);
 void fgp_const_value (wxText& , wxCommandEvent&);
 void fgp_ok_button (wxButton& , wxCommandEvent&);
 void fgp_cancel_button (wxButton&, wxCommandEvent&);
 void cb_gen_dat_call(wxCheckBox&,wxCommandEvent&);
 void cb_gen_ghs_call(wxCheckBox&,wxCommandEvent&);
 void cb_gen_gpp_call (wxCheckBox& o,wxCommandEvent&);
#endif /* mutual exclusion */
